import os
import requests
from tqdm.auto import tqdm
from tqdm.utils import CallbackIOWrapper

from .base import discovery_api, Base
class DatasetManager(Base):

    def upload_file(self, directory_id=None, file_path=None, description=""):
        if type(file_path) != str :
            raise Exception(f"file_path expect `str` but got {type(file_path)}, or please input `file_path='<path>'`")
        elif not os.path.exists(file_path):
            raise Exception(f"file {os.path.abspath(file_path)} not found")
        if directory_id == None:
             raise Exception("please input directory, like `directory=<id>`")
        _res = requests.get(f"{discovery_api}/directory/{directory_id}",  headers=self._jwt_header)
        if _res.status_code == 404:
            raise Exception(f"directory {directory_id} not found or your dose not has permission to access directory")
        f_name = os.path.basename(file_path)
        _res = requests.post(f"{discovery_api}/file/", headers=self._jwt_header,
                             data={
                                 'name': f_name,
                                 'description': f_name,
                                 'directory': int(directory_id),
                                 'size': os.path.getsize(file_path),
                                 'is_use': True
                             }
                )
        if not _res.status_code in [200, 201]:
            txt = ""
            if _res.status_code < 500:
                txt = _res.json()
            raise Exception(f"{_res.status_code}, {txt}")
        data = _res.json()
        file_size = os.stat(file_path).st_size
        with open(file_path, 'rb') as f:
            with tqdm(total=file_size, unit="B", unit_scale=True, unit_divisor=1024) as t:
                wrapped_file = CallbackIOWrapper(t.update, f, "read")
                _res = requests.put(data['url'], data=wrapped_file)
        if _res.status_code == 200:
            _uploaded = True
        else:
            print(_res.content.decode())
            raise Exception("Some thing wrong with datastore")
        data.update({
            'uploaded': _uploaded 
        })
        return data
    
    def download_file(self, file_id=None, download_path=None):
        if file_id == None:
             raise Exception("please input file_id")
        _res = requests.get(f"{discovery_api}/file/{file_id}/",  headers=self._jwt_header)
        if _res.status_code != 200:
            raise Exception(f"File response code {_res}")
        meta = _res.json()
        _res = requests.get(f"{discovery_api}/file/{file_id}/download/",  headers=self._jwt_header)
        if _res.status_code != 200:
            raise Exception("some thing wrong in datastore")
        
        _res = requests.get(_res.json()['url'])
        f_path = os.path.join(self._tmp_path, meta['name'])
        if download_path != None:
            if (not os.path.exists(download_path)) or (not os.path.isdir(download_path)):
                raise Exception("`download_path` is not existx or is not directory")
            else:
                f_path = os.path.join(self._tmp_path, meta['name'])
            
        
        total_size_in_bytes= int(_res.headers.get('content-length', 0))
        print(total_size_in_bytes)
        block_size = 1024 #1 Kibibyte
        progress_bar = tqdm(total=total_size_in_bytes, unit='iB', unit_scale=True)
        
        with open(f_path, 'wb') as f:
            for data in _res.iter_content(block_size):
                progress_bar.update(len(data))
                f.write(data)
        progress_bar.close()
        
        meta.update({
            'download_sucess': os.path.exists(f_path),
            'f_path': f_path
        })
        return meta


import requests
import pandas as pd
import dask.dataframe as dd
from .base import discovery_api, bucket_name, minio_url, Base
from .dataset import DatasetManager
import io

def append_slash(txt):
    if txt[-1] != '/':
        txt += '/'
    return txt

class DataNode(DatasetManager):
    def write(self, ddf=None, df=None, directory=None, name=None, description="", replace=None, profiling=False, **kwargs):
        if (ddf is None) and (df is None):
            raise Exception(f"Please input ddf(dask dataframe) or df(pandas dataframe), got {type(ddf) or type(df)}")
        if (type(ddf) != dd.DataFrame) and (type(df) != pd.DataFrame):
            raise Exception("Invalid type expect ddf(dask dataframe) or df(pandas dataframe)")
        if name == None or type(name) != str:
            raise Exception(f"Please input data name, but got {type(name)}")
        if description=="":
            description = f"data {name}"
            
        name = f'{name}.parquet'
        _res = requests.get(f"{discovery_api}/directory/{directory}/fileExists/?filename={name}", headers=self._jwt_header)
        if _res.status_code == 200:
            replace = False
        elif _res.status_code == 302:
            replace = input(f"File {name} alrady exists, do you want to replace y/n : ").replace(' ', '').lower() == "y"
        
        _res = requests.post(f'{discovery_api}/file/', headers=self._jwt_header,
                                json={
                                    "name": name,
                                    "description": description,
                                    "directory": directory,
                                    "is_use": False,
                                    "replace": replace
                                }
                            )
        if _res.status_code != 201:
            raise Exception(f"can not create directory in discovery {_res.json()}")
        meta = _res.json()
        _df_ddf = ddf if type(ddf) != None else df
        _df_ddf.to_parquet(f"s3://{bucket_name}/{meta['key']}",
            storage_options=self._storage_options,
            **kwargs
        )
        
        # create profiling & data dict
        if profiling:
            requests.get(f"{discovery_api}/file/{meta['id']}/createDatadict/", headers=self._jwt_header)
            requests.get(f"{discovery_api}/file/{meta['id']}/createProfileling/", headers=self._jwt_header)
            
        return {
            'sucess': True,
            'file_id': meta['id'],
            'path': meta['key']
        }
    
    def get_file(self, file_id=None):
        _res = requests.get(f"{discovery_api}/file/{file_id}/", headers=self._jwt_header)
        if _res.status_code != 200:
            txt = _res.json() if _res.status_code < 500 else " "
            raise Exception(f"Some thing wrong, {txt}")
        meta = _res.json()
        
        try:
            response = self.client.get_object(bucket_name=bucket_name, object_name=meta['s3_key'])
        except Exception as e:
            raise e
        else:
            return io.BytesIO(response.data)
        
    
    def read_ddf(self, file_id=None):
        _res = requests.get(f"{discovery_api}/file/{file_id}/", headers=self._jwt_header)
        if _res.status_code != 200:
            txt = _res.json() if _res.status_code < 500 else " "
            raise Exception(f"Some thing wrong, {txt}")
        meta = _res.json()
        meta.update({
            'key': append_slash(meta['s3_key'])
        })
        _f_type = meta['type']['name']
        if _f_type == "parquet":
            return dd.read_parquet(f"s3://{bucket_name}/{meta['key']}", storage_options=self._storage_options)
        elif _f_type == "csv":
            return dd.read_csv(f"s3://{bucket_name}/{meta['key']}", storage_options=self._storage_options)
        return None
        
    def read_df(self, file_id=None):
        _res = requests.get(f"{discovery_api}/file/{file_id}/", headers=self._jwt_header)
        if _res.status_code != 200:
            txt = _res.json() if _res.status_code < 500 else " "
            raise Exception(f"Some thing wrong, {txt}")
        meta = _res.json()
        meta.update({
            'key': append_slash(meta['s3_key'])
        })
        _f_type = meta['type']['name']
        if _f_type == "parquet":
            return pd.read_parquet(f"s3://{bucket_name}/{meta['key']}", storage_options=self._storage_options)
        elif _f_type == "csv":
            return pd.read_csv(f"s3://{bucket_name}/{meta['key']}", storage_options=self._storage_options)
        return None


import os
import requests
import time

from minio import Minio

defind_clear_output = False
try:
    from IPython.display import clear_output
    defind_clear_output = True
except:
    pass

discovery = "https://discovery.data.storemesh.com"

base_discovery_api = "https://api.discovery.data.storemesh.com"
# base_discovery_api = "http://192.168.24.207:8000"
discovery_api = f"{base_discovery_api}/api/v2"

base_minio_url="s3.minio.data.storemesh.com"
# base_minio_url="192.168.24.207:9000"

minio_url=f"https://{base_minio_url}"
bucket_name = 'dataplatform'

class Base:
    def __init__(self, token=None):
        """Init DSM Dataset Manager

        Args:
            jwt_token (str): JWT token from IaM.
        """
        if token is None:
            print(f"Please get token from {discovery}")
            token = input("Your Token : ")
            if defind_clear_output:
                time.sleep(2)
                clear_output()
        if token in [None, '']:
            raise Exception('Please enter your key from dsmOauth')
        self._jwt_header = {
            'Authorization': f'Bearer {token}'
        }
        _res = requests.get(f"{base_discovery_api}/api/v2/account/me/", headers=self._jwt_header)
        if _res.status_code != 200:
            txt = _res.json() if _res.status_code < 500 else " "
            raise Exception(f"Can not connect to DataPlatform, {txt}")
        self.token = token
        self._tmp_path = 'dsm.tmp'
        os.makedirs(self._tmp_path, exist_ok=True)
        _res = requests.get(f"{base_discovery_api}/api/minio/minio-user/me/", headers=self._jwt_header)
        if _res.status_code != 200:
            raise Exception("Can not get minio user")
        data = _res.json()
        self._minio_access = data['access']
        self._minio_secret = data['secret']
        
        # self.client = Minio(
        #     base_minio_url,
        #     access_key=self._minio_access,
        #     secret_key=self._minio_secret,
        #     secure=True
        # )
        # self._storage_options = {
        #     'key': self._minio_access,
        #     'secret': self._minio_secret,
        #     'client_kwargs':{
        #         'endpoint_url': minio_url
        #     }
        # }
        self.client = Minio(
            "192.168.24.207:9000",
            access_key=self._minio_access,
            secret_key=self._minio_secret,
            secure=False
        )
        
        self._storage_options = {
            'key': self._minio_access,
            'secret': self._minio_secret,
            'client_kwargs':{
                'endpoint_url': "http://192.168.24.207:9000"
            }
        }